#!/usr/bin/ruby
#--
# ucert
#
# A  Ruby library for enterprise user account certification / recertification / audit project
#
# Copyright (c)  CMBNY Risk Department
#++

# A server side script designed to automatically update the 'cyber_idm' application's 'ad_users'
#    and 'ad_computers' tables in a scheduled time-table
# Usage:
# =>  ruby idm_ad_reload.rb -h 

require 'optparse'
require 'ostruct'
require "ucert"
require "sequel"
require "yaml"

class CmdOptions
    # Returen an OpenStruct describing the command line options
  def self.parse(args)
    @options = OpenStruct.new # OpenStruct is a structure like hash table
    @options.verbose = false # verbose and banner are key and value
    @options.banner = "Usage: #{__FILE__} -h"
    @version = ["0","1"]    # Program version information
    @last_change = "03/14/2016"
    opt_parser = OptionParser.new do |opts|

      # Boolean switch for the verbose mode switch
      opts.on("-v", "--[no-]verbose", "Verbose Mode") do |v|
        @options.verbose = v
      end

      # Command switch for configuration file
      opts.on("-lYML", "--ldap=YML", "Ldap YML Configuration File") do |l|
        @options.ldap = l
      end

      # Command switch for configuration file
      opts.on("-dYML", "--db=YML", "Database YML Configuration File") do |d|
        @options.db = d
      end

      # Displaying help.
      opts.on("-h", "--help", "Display Help") do |v|
        puts opts
        exit
      end

      # Another typical switch to print the version.
      opts.on_tail("-V","--version", String, "Show Program Version") do
        @options.version = "Program Version: #{@version.join('.')}; Last Modification: #{@last_change}"
        puts @options.version
        exit
      end
    end
    opt_parser.parse!(args)
    puts "Captured Command Line Arguments: #{@options}" if @options.verbose
    return @options
  end
end

##########################################################################
###                              Main                                  ###
##########################################################################
# Command line arguments structure
@options=CmdOptions.parse(ARGV)

ldap_yml = @options.ldap || "/home/liyang138/cyber_idm/config/ldap.yml"
db_yml = @options.db || "/home/liyang138/cyber_idm/config/database.yml"

# Step 1 - Read the AD connecting db string from the 'cyber_idm' configuration file 'ldap.yml'
ldap_env = YAML.load_file(ldap_yml)[ENV['RAILS_ENV'] || 'development']
db_env = YAML.load_file(db_yml)[ENV['RAILS_ENV'] || 'development']

# Step 2 -  Update the local cache database for AD
puts "Update AD local cache by using openldap connector ..."
ny_ad=Ucert::AdTracker.new(:verbose=>@options.verbose)
ny_ad.ldap_connector="openldap"
ny_ad.ldap_connector_id=ldap_env["admin_user"]
ny_ad.ldap_connector_pass=ldap_env["admin_password"]
ny_ad.update_ad_cache("person")
ny_ad.update_ad_cache("computer")
# reload
ny_ad.parse_openldap_cache("person")
ny_ad.parse_openldap_cache("computer")
puts "Done update AD cache."

# Step 3 - connect to the 'cyber_idm' DB
db = Sequel.connect(:adapter => db_env["adapter"], :user => db_env["username"], :password => db_env["password"], :host => db_env["host"] , :database => db_env["database"])

# Step 4 - Reload the 'ad_users' table
puts "\n\nReload ad_users table ..."
db[:ad_users].truncate
ad_user_table = db[:ad_users]
my_users = Hash.new
ny_ad.ad_person_records.keys.map do |my_dn|
  my_sam_id=ny_ad.get_dn_attribute("person",my_dn,"sAMAccountName")
  my_email=ny_ad.get_dn_attribute("person",my_dn,"mail")
  my_name=ny_ad.get_cn(my_dn)
  my_dept=ny_ad.get_dn_attribute("person",my_dn,"department")
  my_acct_code=ny_ad.get_dn_attribute("person",my_dn,"userAccountControl")
  my_acct_status=ny_ad.cntl_code_2_property_flag(my_acct_code)
  my_workstations=ny_ad.get_dn_attribute("person",my_dn,"userWorkstations")
  my_membership=ny_ad.get_dn_attributes("person",my_dn,"memberOf")
  my_users[my_dn] = Hash.new
  my_users[my_dn]['DN'] = my_dn
  my_users[my_dn]['NAME'] = my_name
  my_users[my_dn]['SAM'] = my_sam_id
  my_users[my_dn]['EMAIL'] = my_email
  my_users[my_dn]['DEPARTMENT'] = my_dept
  my_users[my_dn]['ACCOUNT_STATUS'] = my_acct_status
  my_users[my_dn]['WORKSTATION'] = my_workstations
  my_users[my_dn]['MEMBERSHIP'] = my_membership
end
my_users.values.map do |record|
  puts "Insert record into ad_users table: #{record}" if @verbose
  if ad_user_table.insert(:dn => record['DN'].to_s, :name => record['NAME'].to_s, :sam => record['SAM'].to_s, :email => record['EMAIL'].to_s, :department => record['DEPARTMENT'].to_s, :account_status => record['ACCOUNT_STATUS'].to_s, :work_station_info => record['WORKSTATION'].to_s, :membership => record['MEMBERSHIP'].to_s)
    puts "Success!" if @verbose
  else
    puts "Insert fail. " if @verbose
  end
end
puts "Done reload ad_users."

# Step 5 - Reload the 'ad_computers' table
puts "\n\nReload ad_computers table ..."
db[:ad_computers].truncate
ad_computer_table = db[:ad_computers]
my_computers = Hash.new
ny_ad.ad_computer_records.keys.map do |my_dn|
  my_sam_id=ny_ad.get_dn_attribute("computer",my_dn,"sAMAccountName")
  my_os=ny_ad.get_os_info(my_dn)
  my_host=ny_ad.get_dn_attribute("computer",my_dn,"dNSHostName")
  if ny_ad.known_hosts.key?(my_host)
    my_ip=ny_ad.known_hosts[my_host]
  else
    my_ip=ny_ad.nslookup(my_host)
    ny_ad.known_hosts[my_host]=my_ip
  end
  my_created=ny_ad.get_dn_attribute("computer",my_dn,"whenCreated")
  my_computers[my_dn] = Hash.new
  my_computers[my_dn]['DN'] = my_dn
  my_computers[my_dn]['SAM'] = my_sam_id
  my_computers[my_dn]['OS'] = my_os
  my_computers[my_dn]['HOST'] = my_host
  my_computers[my_dn]['IP'] = my_ip
  my_computers[my_dn]['CREATED'] = my_created
end
my_computers.values.map do |record|
  puts "Insert record into ad_computers table: #{record}" if @verbose
  if ad_computer_table.insert(:dn => record['DN'].to_s, :sam => record['SAM'].to_s, :os => record['OS'].to_s, :host => record['HOST'].to_s, :ip => record['IP'].to_s, :created => record['CREATED'].to_s)
    puts "Success!" if @verbose
  else
    puts "Insert fail." if @berbose
  end
end
puts "Done reload ad_computers."

# Step 6 - clean up
db.disconnect
ny_ad = nil
